from setuptools import setup

setup(name='willacy_cardGames',
      version='0.3',
      description='Card game Old Maid',
      packages=['willacy_cardGames'],
      author="Andrew Willacy",
      email="andrewwillacy@gmail.com",
      zip_safe=False)
